/*
 * benchmark program:   complex_update_ext.c
 * 
 * benchmark suite:     DSP-kernel
 * 
 * description:         complex_update - filter benchmarking
 * 
 * This program performs a complex update of the form D = C + A*B,
 * where A, B, C and D are complex numbers .
 * 
 *          A = Ar + j Ai
 *          B = Br + j Bi
 *          C = Cr + j Ci
 *          D = C + A*B =   Dr  +  j Di
 *                      =>  Dr = Cr + Ar*Br - Ai*Bi
 *                      =>  Di = Ci + Ar*Bi + Ai*Br
 * 
 * reference code:      target assembly
 * 
 * f. verification:     simulator based
 * 
 *  organization:        Aachen University of Technology - IS2 
 *                       DSP Tools Group
 *                       phone:  +49(241)807887 
 *                       fax:    +49(241)8888195
 *                       e-mail: zivojnov@ert.rwth-aachen.de 
 *
 * author:              Juan Martinez Velarde
 * 
 * history:             11-05-94 creation (Martinez Velarde)(complex_multiply.c)
 *                      22-02-95 language extensions (Harald L. Schraut)
 *
 *                      $Author: schraut $
 *                      $Date: 1995/02/22 13:09:42 $
 *                      $Revision: 1.1 $ 
 */

#define STORAGE_CLASS register
#define TYPE  int

void
pin_down(TYPE *p)
{
   
  *p++ = 0 ; 
  *p   = 0 ; 

}


TYPE
main()
{
  static TYPE pm A[2] = { 2,1 } ;
  static TYPE dm B[2] = { 2,5 } ;
  static TYPE dm C[2] = { 3,4 } ; 
  static TYPE dm D[2] = { 0,0 } ; 
  
  STORAGE_CLASS TYPE pm *p_a = &A[0] ;
  STORAGE_CLASS TYPE dm *p_b = &B[0] ;
  STORAGE_CLASS TYPE dm *p_c = &C[0] ;
  STORAGE_CLASS TYPE dm *p_d = &D[0] ;
  
  pin_down(&D[0]) ; 
  
  START_PROFILING;
	  
  *p_d    = *p_c++ + *p_a++ * *p_b++ ;
  *p_d++ -=          *p_a   * *p_b-- ; 

  *p_d    = *p_c   + *p_a-- * *p_b++ ; 
  *p_d   +=          *p_a   * *p_b ; 
   
  END_PROFILING;
  
  pin_down(&D[0]) ; 
  
  return(0)  ; 
}
