;/*
; * benchmark program:   complex_update_arc1.asm
; * 
; * benchmark suite:     DSP-kernel
; * 
; * description:         complex_update - filter benchmarking
; * 
; * This program performs a complex update of the form D = C + A*B,
; * where A, B, C and D are complex numbers .
; * 
; *          A = Ar + j Ai
; *          B = Br + j Bi
; *          C = Cr + j Ci
; *          D = C + A*B =   Dr  +  j Di
; *                      =>  Dr = Cr + Ar*Br - Ai*Bi
; *                      =>  Di = Ci + Ar*Bi + Ai*Br
; * 
; * reference code:      target assembly
; * 
; * f. verification:     simulator based
; * 
; *  organization:        Aachen University of Technology - IS2 
; *                       DSP Tools Group
; *                       phone:  +49(241)807887 
; *                       fax:    +49(241)8888195
; *                       e-mail: zivojnov@ert.rwth-aachen.de 
; *
; * author:              Juan Martinez Velarde
; * 
; * history:             25-5-94 creation (Martinez Velarde)
; * 
; */
	section complex_update_arc1
	opt	so,nomd,cc

        org     x:
V__1:	dc      3
	dc      2
	dc      1
V__2:	dc	2
	dc	5
	dc	4
V__d:   dc      0
	dc	0

LC0	dc	'dr',$3D,$25,'i',$0A,$0A,$00
LC1	dc	'di',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,x:(r2)+
        move 	#V__1,r0
        move 	#V__2,r3
	move	#V__d,r1
	clr	a
	clr	b
	nop

	START_PROFILING:

        move    x:(r0)+,a	   		;cr
	move	x:(r0)+,y1   x:(r3)+,x1		;ar,br
	imac	y1,x1,a   			;cr+ar*br
	move	x:(r0)+,y0   x:(r3)+,x0		;ai,bi
	impy	y0,x0,b							
	sub	b,a	     x:(r3)+,b		;-ai*bi,ci
	imac	y1,x0,b	   			;ci+ar*bi
	imac	y0,x1,b				;+ai*br
	move	a,x:(r1)+			;store dr
	move	b,x:(r1)+			;store di

	END_PROFILING:
	
	clr 	b
	move	x:(r3)+,b
	move	#LC0,r1
	move	b,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move 	(r2)-
	move	(r2)-

	clr 	b
	move	x:(r3),b
	move	#LC1,r1
	move	b,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move 	(r2)-
	move	(r2)-

; **************************************************

	move	x:-(r2),ssh
	rts
	endsec