;/*
; * benchmark program:   complex_update_arc2.asm
; * 
; * benchmark suite:     DSP-kernel
; * 
; * description:         complex_update - filter benchmarking
; * 
; * This program performs a complex update of the form D = C + A*B,
; * where A, B, C and D are complex numbers .
; * 
; *          A = Ar + j Ai
; *          B = Br + j Bi
; *          C = Cr + j Ci
; *          D = C + A*B =   Dr  +  j Di
; *                      =>  Dr = Cr + Ar*Br - Ai*Bi
; *                      =>  Di = Ci + Ar*Bi + Ai*Br
; * 
; * reference code:      Dr.Bub/56100/source.1/B.2.9 (modified)
; * 
; * f. verification:     simulator based
; * 
; *  organization:        Aachen University of Technology - IS2 
; *                       DSP Tools Group
; *                       phone:  +49(241)807887 
; *                       fax:    +49(241)8888195
; *                       e-mail: zivojnov@ert.rwth-aachen.de 
; *
; * author:              Juan Martinez Velarde
; * 
; * history:             25-5-94 creation (Martinez Velarde)
; * 
; */
	section complex_update_arc2
	opt	so,nomd,cc

        org     x:
V__1:	dc      3
	dc      2
	dc      1
V__2:	dc	2
	dc	5
	dc	4
V__d:   dc      0
	dc	0

LC0	dc	'dr',$3D,$25,'i',$0A,$0A,$00
LC1	dc	'di',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,x:(r2)+

        move 	#V__1,r0
        move 	#V__2,r3
	move	#V__d,r1
	clr	a
	clr	b
	nop

	START_PROFILING:

        move    x:(r0)+,a0 		   	 ;cr
	asl	a
	move	x:(r0)+,y1 x:(r3)+,x1		 ;ar,br
	mac	y1,x1,a    x:(r0)+,y0 x:(r3)+,x0 ;cr+ar*br,ai,bi
	mac	-y0,x0,a   x:(r3)+,b0 		 ;-ai*bi,ci
	asr	a
	asl	b	   a0,x:(r1)+		 ;store dr
	mac	y1,x0,b				 ;ci+ar*bi
	mac	y0,x1,b				 ;+ai*br
	asr	b
	move	b0,x:(r1)+			 ;store di

	END_PROFILING:

; **************************************************

	move	#LC0,r1
	move	#V__d,r3
	clr 	b
	move	x:(r3)+,b0
	move	b0,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move 	(r2)-
	move	(r2)-

	move	#LC1,r1
	clr 	b
	move	x:(r3)+,b0
	move	b0,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move 	(r2)-
	move	(r2)-

; **************************************************

	move	x:-(r2),ssh
	rts
	endsec