;/*
; * benchmark program:   complex_update_arc.asm
; * 
; * benchmark suite:     DSP-kernel
; * 
; * description:         complex_update - filter benchmarking
; * 
; * This program performs a complex update of the form D = C + A*B,
; * where A, B, C and D are complex numbers .
; * 
; *          A = Ar + j Ai
; *          B = Br + j Bi
; *          C = Cr + j Ci
; *          D = C + A*B =   Dr  +  j Di
; *                      =>  Dr = Cr + Ar*Br - Ai*Bi
; *                      =>  Di = Ci + Ar*Bi + Ai*Br
; * 
; * reference code:      target assembly
; * 
; * f. verification:     simulator based
; * 
; *  organization:        Aachen University of Technology - IS2 
; *                       DSP Tools Group
; *                       phone:  +49(241)807887 
; *                       fax:    +49(241)8888195
; *                       e-mail: zivojnov@ert.rwth-aachen.de 
; *
; * author:              Juan Martinez Velarde
; * 
; * history:             11-5-94 creation (Martinez Velarde)
; * 
; */
	section complex_update_arc
	opt	so,nomd,cc

        org     x:$0
V__a:   dc      2
        dc      1

V__c:   dc      10
        dc      10

        org     y:$0
V__b:   dc      2
        dc      5
V__d:   dc      0
        dc      0

LC0	dc	'dr',$3D,$25,'i',$0A,$0A,$00
LC1	dc	'di',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,y:(r6)+

        move 	#V__a,r3
	move	#V__d,r2
        move 	#V__c,r1
	move	#V__b,r4
	clr	a
	clr	b
	nop

	START_PROFILING:

        move              x:(r1)+,a	           ;a=cr
	move	a,a0
	asl	a	  x:(r1)+,b                ;b=ci
	move	b,b0  
	asl	b	  x:(r3)+,x1  y:(r4)+,y1   ;x1=ar,y1=br
        mac     y1,x1,a   x:(r3)+,x0  y:(r4)+,y0   ;a+=ar*br,x0=ai,y0=bi
        mac    -y0,x0,a                            ;a-=ai*bi
	mac	y1,x0,b                            ;b+=ar*bi
	mac     y0,x1,b                            ;b+=ai*br
	asr 	a                                  ;
	asr 	b	a0,y:(r2)+                  ;store a
       	move            b0,y:(r2)+                  ;store b

	END_PROFILING:

; **************************************************
	
	clr	a
	clr	b

	move	#V__d,r2

	move	#LC0,r1
	move	y:(r2)+,a0
	move	a0,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-

	move	#LC1,r1
	move	y:(r2),b0
	move	b0,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-

; **************************************************

	move	y:-(r6),ssh
	rts
	endsec