; /*
;  * benchmark program:   dot_product_arc.asm
;  * 
;  * benchmark suite:     DSP-kernel
;  * 
;  * description:         dot_product - filter benchmarking
;  *
;  *     [2x1] vectors on the DSP56100. 
;  *
;  *     Vector a is in X memory, a = (a1,a2)
;  *     vector b is in X memory, b = (b1,b2)T
;  *     the result z is stored in X memory. z = a1*b1 + a2*b2
;  *
;  *
;  *   X Memory             X Memory
;  *
;  *     | a2 |             |  b2 |
;  *  |->| a1 |          |->|  b1 |
;  *  r0                 r3      
;  * 
;  *  |->| z  |       
;  *  r1
;  * 
;  * reference code:      Dr.Bub/56000/bench/7-56.asm (modified for 56156)
;  * 
;  * f. verification:     with printf function
;  * 
;  *  organization:        Aachen University of Technology - IS2 
;  *                       DSP Tools Group
;  *                       phone:  +49(241)807887 
;  *                       fax:    +49(241)8888195
;  *                       e-mail: zivojnov@ert.rwth-aachen.de 
;  *
;  * author:              Juan Martinez Velarde
;  * 
;  * history:             15-7-94 creation and profiling (Martinez Velarde)
;  * 
;  */

	section dot_product_arc2
	opt	so,nomd,cc

veca	equ	$0

        org     x:veca
V__a:   dc      $2
        dc      $2

V__b:   dc      $5
        dc      $2
z:      dc      0

LC0
	dc	'z',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,x:(r2)+
        move 	#veca,r0                   ;point to vectors a,b
	move	#veca+2,r3
        move 	#z,r1                      ;point to z
	nop

	START_PROFILING:

        move            x:(r0)+,y1  x:(r3)+,x1     ;load a1,b1
	impy  x1,y1,a    
	move	x:(r0)+,y0  x:(r3)+,x0     ;a=a1*b1,load a2,b2
        imac  y0,x0,a                    	   ;a=+a2*b2
        move            a,x:(r1)        	   ;store z

	END_PROFILING:

; **************************************************
	clr	a
	move	#LC0,r1
        move 	#z,r0                     ;point to z
	move	x:(r0),a1
	move	a1,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move 	(r2)-
	move	(r2)-
; **************************************************
	move	x:-(r2),ssh
	rts
	endsec