; /*
;  * benchmark program:   dot_product_arc.asm
;  * 
;  * benchmark suite:     DSP-kernel
;  * 
;  * description:         dot_product - filter benchmarking
;  *
;  *     [2x1] vectors on the DSP56000. 
;  *
;  *     Vector a is in X memory, 
;  *     vector b is in Y memory, 
;  *     the result z is stored in X memory.
;  *
;  *
;  *   X Memory                     Y Memory
;  *
;  *  |  | x2 |          |->|  y2 |
;  *  |->| x1 |          |->|  y1 |
;  *  r0 |    |          r0 |     |
;  * 
;  *  |->| z  |              
;  * 
;  *
;  * Note: the previous assumes that all immediate addressing is
;  * immediate short, i.e. all data is in internal memory.
;  *
;  * 
;  * reference code:      Dr.Bub/56000/bench/7-56.asm
;  * 
;  * f. verification:     with printf function
;  * 
;  *  organization:        Aachen University of Technology - IS2 
;  *                       DSP Tools Group
;  *                       phone:  +49(241)807887 
;  *                       fax:    +49(241)8888195
;  *                       e-mail: zivojnov@ert.rwth-aachen.de 
;  *
;  * author:              Juan Martinez Velarde
;  * 
;  * history:             10-5-94 creation and profiling (Martinez Velarde)
;  * 
;  */

	section dot_product
	opt	so,nomd,cc

veca	equ	$0

        org     x:veca
V__a:   dc      $2000
        dc      $2000

        org     y:veca
V__b:   dc      $5000
        dc      $2000
z:      dc      0

LC0
	dc	'z',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,y:(r6)+
        move 	#<veca,r0                   ;point to vectors a,b
        move 	#<z,r2                      ;point to z
	nop

	START_PROFILING:

        move            l:(r0)+,x       ;load first elements.
        mpy  x0,x1,a    l:(r0)+,x       ;a1*b1
        macr x0,x1,a                    ;+a2*b2
	asr a				;shift right one
        move            a,x:(r2)        ;--> z

	END_PROFILING:

; **************************************************
	clr	a
	move	#LC0,r1
	move	x:(r2),a
	move	a1,y:(r6)+
	move	r1,y:(r6)+
	jsr	Fprintf
	move 	(r6)-
	move	(r6)-
; **************************************************
	move	y:-(r6),ssh
	rts
	endsec