{ 
  DIT FFT IDS Main program                               DIT_16_IDS.DSP

    benchmark program  : dit_16_ids.dsp
 
    benchmark suite    : DSP-kernel

    description        : benchmarking of an integer FFT (16 point)
                         Radix 2, In-place, DIT Complex FFT
			 Input Data must be Scaled 

     This program computes a 16-point DFT of data values stored data
     buffers using DIT FFT algorithm.  The input data is assumed to be
     complex.  The real and imaginary input data are stored in inputreal
     and inputimag buffers, respecively.  The input data is read from the
     disk files INPUTREAL_16.DAT and INPUTIMAG_16.DAT. The real and imaginary
     parts of twiddle factors are read from disk files TWID_REAL_16.DAT and 
     TWID_IMAG_16.DAT, respectively.  The output DFT values are available in
     data memory locations inplacereal and inplaceimag.

     Calling Parameters
        Sequentially ordered input data in inplacereal and inplaceimag
	0x000 inplacereal address
	0x400 inplaceimag address

     Return Values
         inplacereal = Real FFT results in sequential order
         inplaceimag = Imaginary FFT results in sequential order

     Altered Registers
         I0,I1,I2,I3,I4,I5,L0,L1,L2,L3,L4,L5
         M0,M1,M2,M3,M4,M5
         AX0,AX1,AY0,AY1,AR,AF
         MX0,MX1,MY0,MY1,MR,SB,SE,SR,SI

     Altered Memory
         inplacereal, inplaceimag, groups, node_space,
         bflys_per_group, blk_exponent

     organization       : Aachen University of Technology - IS2
     	                : DSP Tools Group
        	        : phone   : +49(241)807887
                        : fax     : +49(241)8888195
                        : e-mail  : zivojnov@ert.rwth-aachen.de

     References
        dsp laboratory
        (1) /tools/home1/dsp/adi21xx/21xx/dsp_laboratory/chap8/8_3_2
	(2) Digital Signal Processing Laboratory using the 
	    adsp-2101 microcomputer, chapter 8, pp.212.

     Adaption
        dit_brev program adapted for scrambling real and imaginary data
	author : 	Juan Martinez Velarde
	history:	creation and profiling, 11-Mar-1994
}

.MODULE/ABS=0            dit_16_ids;
.CONST                   N=16, N_div_2=8, logN=4;{Const. for 16 points}
.CONST			 mod_value=0x400;
.VAR/PM/RAM/CIRC         twid_real [N_div_2];
.VAR/PM/RAM/CIRC         twid_imag [N_div_2];
.VAR/DM/RAM/ABS=0x0      inplacereal [N] ; 
.VAR/DM/RAM/ABS=0x400    inplaceimag [N] ; {fix locations}
.VAR/DM/RAM/ABS=0x800    inputreal[N],inputimag [N], padding [4];
.VAR/DM/RAM              groups, bflys_per_group, node_space, blk_exponent;

.INIT      twid_real: <twid_real_16.dat>;
.INIT      twid_imag: <twid_imag_16.dat>;
.INIT      inputreal: <inputreal_16.dat>;
.INIT      inputimag: <inputimag_16.dat>;
.INIT      groups: N_div_2;
.INIT      bflys_per_group: 1;
.INIT      node_space: 1;
.INIT      blk_exponent: 0;
.INIT      padding: 0,0,0,0;                     {Zeros after inplaceimag}

.GLOBAL    inplacereal, inplaceimag;
.GLOBAL    inputreal, inputimag;
.GLOBAL    twid_real, twid_imag;
.GLOBAL    groups, bflys_per_group, node_space, blk_exponent;

START_PROFILING:

{ Scramble inputreal and inputimag data}

scramble:   I4=^inputreal;     {I4 sequentially ordered data}
            I0=^inplacereal;   {I0 scrambled data, inplacereal should be at 0x0}
            M4=1;
            M0=mod_value;      {M0=modifier for reversing N bits}
            L4=0;
            L0=0;

            CNTR = N;
            ENA BIT_REV;       {Enable bit-reversed outputs on DAG1}
            DO brevreal UNTIL CE;
            AY1=DM(I4,M4);     {Read sequentially ordered data}
brevreal:   DM(I0,M0)=AY1;     {Write data in bit-reversed location}
            DIS BIT_REV;       {Disable bit-reverse}
			       {Values for M0,M4,L0 and L4 invariant} 
            I4=^inputimag;     {I4 sequentially ordered data}
            I0=0x8;	       {I0 scrambled data}
                               {inplaceimag should be at 0x400}
            CNTR = N;
            ENA BIT_REV;       {Enable bit-reversed outputs on DAG1}
            DO brevimag UNTIL CE;
            AY1=DM(I4,M4);     {Read sequentially ordered data}
brevimag:   DM(I0,M0)=AY1;     {Write data in bit-reversed location}
            DIS BIT_REV;       {Disable bit-reverse}

fft_ids:    
         CNTR=logN;                {Initialize stage counter}
         M0=0;
         M1=1;
	 M5=1;
         L1=0;
         L2=0;
         L3=0;
         L4=%twid_real;
         L5=%twid_imag;
         L6=0;

         DO stage_loop UNTIL CE;       {Compute all stages in FFT}

             I0=^inplacereal;          {I0 x0 in 1st grp of stage}
             I2=^inplaceimag;          {I2 y0 in 1st grp of stage}
             SB=-2 ;                   {SB to detect data > 14 bits}
             SI=DM(groups);
             CNTR=SI;                  {CNTR = group counter}
             M4=SI;                    {M4=twiddle factor modifier}
             M2=DM(node_space);        {M2=node space modifier}             
	     M6=DM(node_space);        {M6=node space modifier}

             I1=I0;
             MODIFY(I1,M2);            {I1 x1 of 1st grp in stage}
             I3=I2;
             MODIFY(I3,M2);            {I3 y1 of 1st grp in stage}
             I6=^inplaceimag;          {I6 y0 in 1st grp of stage}
             MODIFY(I6,M6);            {I6 y1 of 1st grp in stage}

             DO group_loop UNTIL CE;
                 I4=^twid_real;        {I4  C of W0}
                 I5=^twid_imag;        {I5  (-S) of W0}
                 CNTR=DM(bflys_per_group);  {CNTR = butterfly counter}

                 MY0=PM(I4,M4),MX0=DM(I1,M0); 		{MY0=C,MX0=x1}
                 MY1=PM(I5,M4);		      		{MY1=-S}
	     
                 DO bfly_loop UNTIL CE;

		 	MR=MX0*MY0(SS),MX1=DM(I6,M5) ;		{MR=x1(C),MX1=y1}
 		 	MR=MR-MX1*MY1(RND),AY0=DM(I0,M0);	{MR=x1(C)-y1(-S),AY0=x0}
  		 	AR=MR1+AY0,AY1=DM(I2,M0) ;		{AR=x0+[x1(C)-y1(-S)],AY1=y0}
		 	AR=AY0-MR1,DM(I0,M1)=AR ;		{AR=x0-[x1(C)-y1(-S)]}
		  	              				{x0'=x0+[x1(C)-y1(-S)]}	
		 	MR=MX0*MY1(SS),DM(I1,M1)=AR ;		{MR=x1(-S)}
								{x1'=x0-[x1(C)-y1(-S)]}
		 	MR=MR+MX1*MY0(RND),MX0=DM(I1,M0),MY1=PM(I5,M4) ; 
								{MR=x1(-S)+y1(C)]}
								{MX0=next x1,MY1=next(-S)}
   			AR=AY1-MR1,MY0=PM(I4,M4) ;		{AR=y0-[x19-S)+y1(C)]}
								{MY0=next C}
			AR=MR1+AY1,DM(I3,M1)=AR;		{AR=y0+[x1(-S)+y1(C)]}
								{y1'=y0-[x1(-S)+y1(C)]}
bfly_loop:		DM(I2,M1)=AR ;				{y0'=y0+[x1(-S)+y1(C)]}

                 MODIFY(I0,M2);        {I0 1st x0 in next group}
                 MODIFY(I1,M2);        {I1 1st x1 in next group}
                 MODIFY(I2,M2);        {I2 1st y0 in next group}
                 MODIFY(I3,M2);        {I3 1st y1 in next group}
group_loop:      MODIFY(I6,M6);        {I6 1st y1 in next group}

	     SI=DM(bflys_per_group);
             SR=ASHIFT SI BY 1(LO);
             DM(node_space)=SR0;       {node_space=node_space x 2}
             DM(bflys_per_group)=SR0;  {bflys_per_group=bflys_per_group x 2}
             SI=DM(groups);
             SR=ASHIFT SI BY -1(LO);
stage_loop:  DM(groups)=SR0;           {groups=groups / 2}

END_PROFILING:

	    TRAP;                                   {Halt program}
.ENDMOD;

