{ 
  Bit-Reverse (Scramble) Subroutine                            DIT_BREV.DSP

     Calling Parameters
        Sequentially ordered input data in inputreal and inplaceimag
	0x000 inputreal address
	0x400 inputimag address

     Return Values
        Scrambled input data in inplacereal and inplaceimag

     Altered Registers
        I0,I4,M0,M4,AY1

     Altered Memory
        inplacereal and inplaceimag

     References
        dsp laboratory
        (1) /tools/home1/dsp/adi21xx/21xx/dsp_laboratory/chap8/8_3_2
	(2) Digital Signal Processing Laboratory using the 
	    adsp-2101 microcomputer, chapter 8, pp.212.

     Adaption
        dit_brev program adapted for scrambling real and imaginary data
	author : Juan Martinez Velarde, 10-Mar-1994
	
}

.MODULE     dit_scramble;
.CONST      N_FFT=1024,mod_value=0x0010;   {Initialize constants}
.EXTERNAL   inputreal, inplacereal;
.EXTERNAL   inputimag, inplaceimag;

.ENTRY      scramble;

scramble:   I4=^inputreal;     {I4 sequentially ordered data}
            I0=^inplacereal;   {I0 scrambled data, inputreal should be at 0x0}
            M4=1;
            M0=mod_value;      {M0=modifier for reversing N_FFT bits}
            L4=0;
            L0=0;
            CNTR = N_FFT;
            ENA BIT_REV;       {Enable bit-reversed outputs on DAG1}
            DO brevreal UNTIL CE;
            AY1=DM(I4,M4);     {Read sequentially ordered data}
brevreal:   DM(I0,M0)=AY1;     {Write data in bit-reversed location}
            DIS BIT_REV;       {Disable bit-reverse}
			       {Values for M0,M4,L0 and L4 invariant} 
            I4=^inputimag;     {I4 sequentially ordered data}
            I0=0x8;	       {I0 scrambled data, inputimag should be at 0x400}
            CNTR = N_FFT;
            ENA BIT_REV;       {Enable bit-reversed outputs on DAG1}
            DO brevimag UNTIL CE;
            AY1=DM(I4,M4);     {Read sequentially ordered data}
brevimag:   DM(I0,M0)=AY1;     {Write data in bit-reversed location}
            DIS BIT_REV;       {Disable bit-reverse}

            RTS;               {Return to calling program}

.ENDMOD;
