*  benchmark program  : c5cx0016.asm
* 
*  benchmark suite    : DSP-kernel
*
*  description        : benchmarking of an integer FFT
*                       Radix 2, In-place, Decimation-in-time Complex FFT Macro
*
*			The code c5cx0016.asm presents a 16 point 
*			Decimation in Time, radix-2, Complex FFT
*
*    PROGRAMSEQUENCE: 0. INITIALISATION FOR FFT/COEFF      
*                     1. INPUT NEW DATA INTO 'INPUT'      
*                     2. CALL SUBROUTINE FFT               
*                     2.1. BITREVERSAL FROM INPUT TO DATA  
*                     2.2. FFT WITH WORK SPACE DATA        
*                     3. OUTPUT THE RESULTS FROM DATA      
*
*    USED  REGISTER: INDX,AR0..AR6,ACCU,PREG,TREG0, PMST, BRCR                
*                    2 Stacklevel, Block B2 for temp variables                
*                                                                             
*    PROGRAM MEMORY:  158 WORDS ('FFTLEN') WITHOUT INITIALIZATION             
*                                                                             
*    COEFFICIENTS  :   16 BITS  (Q15 Format)  SCALING:   1/2^4                
*                                                                             
*    CYCLES	   :  463 CYLCES   =>  23.15 us  16.21 us   11.58 us	      
*			  T-CYCLE  =>	  50 ns     35 ns      25 ns	      
*
*    REFERENCE :    WRITTEN BY: MANFRED CHRIST  REVISION: 2.00   
*		
*                                                                             
*    MEMORY USAGE: FFT PROGRAM    [ 00600H -  0069dH ]                        
*                  INPUT DATA     [ 00800H -  0081fH ]                        
*                  DATA WORKSPACE [ 00100H -  0011fH ]                        
*                  TWIDDLES       [ 00120H -  0012aH ]                        
*                                                                             
*    INPUT DATA AT ADDRESS 0800h-081fh:                                       
*    ----------------------------------                                       
*    THE DATA ARE STORED IN 'input' IN THE SEQUENCE: X(0),X(1),...,X(15)      
*                                                    Y(0),Y(1),...,Y(15)      
*                                                                             
*    OUTPUT DATA AT ADDRESS 0100h-011fh:                                      
*    -----------------------------------                                      
*    THE DATA ARE STORED IN section 'fftdata' IN THE SEQUENCE:                
*    X(0),Y(0),X(1),Y(1),... ... ,X(15),Y(15)                                 
*	
*
*		
*    THE PROGRAM IS BASED ON THE BOOK 'DIGITAL SIGNAL PROCESSING APPLICATIONS'
*    FROM TEXAS INSTRUMENTS P. 69. IT IS OPTIMIZED FOR THE TMS320C5x INCLUDING
*    BIT REVERSAL ADDRESSING MODE.           
*                                                                                
*
*  func. verification : 
*
*
*
*  organization       : Aachen University of Technology - IS2
*                     : DSP Tools Group
*                     : phone   : +49(241)807887
*                     : fax     : +49(241)8888195
*                     : e-mail  : zivojnov@ert.rwth-aachen.de
*
*  author             : Juan Martinez Velarde
*
*  history            : 24-03-94 - assembler reference code creation 
*                       24-03-94 - assembler reference code profiling

         .file      "c5cx0016.asm"
         .title     "0016 point DIT Radix-2, Complex FFT"
         .width     120

N        .set       16    ; NUMBER OF POINTS FOR FFT

         .include init-fft.asm

         .sect    "fftprog"

; FFT CODE WITH BIT-REVERSED INPUT SAMPLES / ARP=AR3 / DP=0

START_PROFILING:

FFT:	 bitrev 	       ; macro to perform the bitreversal
STAGE1:  combo5x	       ; FFT CODE for STAGES 1 and 2
STAGE3:  stage3   2	       ; FFT CODE FOR STAGE 3  / ARP=AR2
STAGE4:  laststag 4,16	       ; FFT CODE FOR STAGE 4  / ARP=AR2

END_PROFILING:


FFTLEN:  .set	  $-FFT
         .end
