; **************************************************
; /*
;  * benchmark program:   fir2dim_arc.asm
;  * 
;  * benchmark suite:     DSP-kernel
;  *
;  * description:         fir 2 dimensional - filter benchmarking
;  * 
;  * The two dimensional FIR uses a 3x3 coefficient mask:
;  *
;  * The image is an array of NxN pixels. To provide boundary conditions
;  * for the FIR filtering, the;image is surrounded by a set of zeros such
;  * that the image is actually stored as a (N+2)x(N+2) array, i.e.
;  *
;  *          <--ARRAYDIM-->  
;  *         |0 0 0 .... 0 0| A
;  *         |0 x x .... x 0| |
;  *         |0 x x .... x 0| ARRAY_
;  *         |0 image area 0| DIM
;  *         |0 x x .... x 0| |
;  *         |0 x x .... x 0| |
;  *         |0 0 0 .... 0 0| V
;  * 
;  *
;  * The image (with boundary) is stored in row major storage. The first
;  * element of the array image is image(1,1) followed by image(1,2). The
;  * last element of the first row is image(1,N+2);followed by the beginning
;  * of the next column image(2,1). 
;  *
;  * Image(1,1) maps to index 0, image(1,N+2) maps to index N+1,
;  * Image(2,1) maps to index N+2 (row major storage).
;  *
;  *
;  *  r0 -> image(n,m)           image(n,m+1)           image(n,m+2)
;  *  r1 -> image(n+N+2,m)       image(n+N+2,m+1)       image(n+N+2,m+2)
;  *  r2 -> image(n+2*(N+2),m)   image(n+2*(N+2),m+2)   image(n+2*(N+2),m+3)
;  *  r3 -> FIR coefficients
;  *  b  -> output image
;  *
;  * 
;  * reference code:      Dr.Bub/56100/source/B.2.23 N Point 3x3 2-D FIR Convolution (modified)
;  * 
;  * f. verification:     with printf function
;  * 
;  *  organization:        Aachen University of Technology - IS2 
;  *                       DSP Tools Group
;  *                       phone:  +49(241)807887 
;  *                       fax:    +49(241)8888195
;  *                       e-mail: zivojnov@ert.rwth-aachen.de 
;  *
;  * author:              Juan Martinez Velarde
;  * 
;  * history:             19-7-94 creation and profiling (Martinez Velarde)
;  * 
;  */

	section fir2dim_arc
	opt so,nomd,cc


n	equ	4 	; number of rows = columns of image

	org	x:$40

image:  dc 0,0,0,0,0,0
	dc 0,1,1,1,1,0
	dc 0,1,1,1,1,0
	dc 0,1,1,1,1,0
	dc 0,1,1,1,1,0
	dc 0,0,0,0,0,0

output: dc 0,0,0,0 ; =>  28 39 39 24
	dc 0,0,0,0 ; =>  33 45 45 27 
	dc 0,0,0,0 ; =>  33 45 45 27
	dc 0,0,0,0 ; =>  16 21 21 12

mask:	dc	1 	; c11
	dc	2 	; c12
	dc	3	; c13
	dc	4	; c21
	dc	5	; c22
	dc	6	; c23
	dc	7	; c31
	dc	8	; c32
	dc	9	; c33

ST0	dc	$25,'i',$0A,$0A,$00

ST1	dc	$25,'i',$0A,$00

ST2	dc	$25,'i',$0A,$0A,$00

ST3	dc	'a=',$25,'i',$0A,$0A,$00


stacksave dc	1
tmpr1	  dc	1
tmpr2	  dc	1
tmpr3	  dc	1

	org	p:$40
	global	Fmain
Fmain
	move	ssh,x:(r2)+	; save ssh

; **** print image to be filtered

;	move	#image,r0
;	move	#ST1,r1
;	do	#((n+2)*(n+2)),INPUTLOOP
;	move	x:(r0)+,a
;	move	a,x:(r2)+
;	move	r1,x:(r2)+
;	jsr	Fprintf
;	move	(r2)-
;	move	(r2)-
;
;INPUTLOOP
;
;	jmp	END	

	move	r2,x:(stacksave)     ; save stack temporaly

        move    #mask,r3              ;pt to coef.
        move    #-8,n3
        move    #image,r0             ;top boundary
        move    #image+n+2,r1         ;left of first pixel
        move    #image+2*(n+2),r2     ;left of 2nd row 
        move    #output,b            ;output image
	move	#-1,n0
	move	#2,y1
        move    n0,n1                 ;adjust
        move    n1,n2

ASM_APP_0
	START_PROFILING:
ASM_APP_1

        move              x:(r0)+,y0	x:(r3)+,x0    ;y0=im(1,1),x0=c11   
        do      #n,rows
        do      #n,cols
        mpy     y0,x0,a   x:(r0)+,y0    x:(r3)+,x0    ; im(1,1)*c11
        mac     y0,x0,a   x:(r0)+n0,y0  x:(r3)+,x0    ;+im(1,2)*c12
        mac     y0,x0,a   x:(r1)+,y0    x:(r3)+,x0    ;+im(1,3)*c13
        mac     y0,x0,a   x:(r1)+,y0    x:(r3)+,x0    ;+im(2,1)*c21
        mac     y0,x0,a   x:(r1)+n1,y0  x:(r3)+,x0    ;+im(2,2)*c22
        mac     y0,x0,a   x:(r2)+,y0    x:(r3)+,x0    ;+im(2,3)*c23
        mac     y0,x0,a   x:(r2)+,y0    x:(r3)+,x0    ;+im(3,1)*c31
        mac     y0,x0,a   x:(r2)+n2,y0  x:(r3)+n3,x0  ;+im(3,2)*c32
        mac     y0,x0,a   x:(r0)+,y0    x:(r3)+,x0    ;+im(3,3)*c33
	asr	a
        move              a0,x:(b1)
	inc24	b
cols
; adjust pointers for frame boundary

	move	x:(r0)+,x1	
	move	(r3)-
	move	x:(r0)+,y0	x:(r3)+,x0	; take next values
	move	x:(r1)+,x1
	move	x:(r1)+,x1
	move	x:(r2)+,x1
	move	x:(r2)+,x1

rows

ASM_APP_2
	END_PROFILING:
ASM_APP_3


	move	x:(stacksave),r2

; **** print filtered image

	move	#output,r0
	move	#ST0,r1
	do	#n*n,L2
	move	x:(r0)+,a0
	move	a0,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move	(r2)-
	move	(r2)-
L2

END:
	move	x:-(r2),ssh
	rts
	endsec

