{ Cascaded Biquad IIR Filter Subroutine                          BIQ_N_ARC.DSP

    This program implements a single precision Cascade Form IIR filter
    structure using subroutine biquad_sub.

    Calling Parameters
         SR1 = input sample
         I0 --> delay line buffer for x(n-2), x(n-1), y(n-2), y(n-1)
         L0 = 0
         I1 --> list of scale factors for each biquad section
         L1 = 0 (in the case of a single biquad)
         L1 = number of biquad sections
         I4 --> scaled coefficients b2,b1,b0,a2,a1,  b2,b1,b0,a2,a1, ...
         L4 = 2.5 * filter order --or-- 5 * number of biquad sections
         M0,M4 = 1
         M1 = -3
         M2 = 1   (in the case of multiple biquads)
         M2 = 0   (in the case of a single biquad)
         M3 = (1 - length of delay line buffer)
         CNTR = number of biquad sections

    Return Values
         SR1 = output sample
         I0 --> inside delay line buffer
         I1 --> top of scale factor list
         I4 --> top of coefficients

    Altered Registers
         MX0,MX1,MY0,MR,SE,SR

    Computation Time
         8 * number of biquad sections + 5 + 5 cycles
    All coefficients and data values are assumed to be in 1.15 format.
}

.MODULE BIQ_N_ARC; 
.CONST  N = 4;             { number of biquad sections, example: 5 }
.CONST  N_x_5 = 20;        { number of biquad sections times five }
.VAR/DM/ABS=0x3400 delayline[4]; {this is scratchpad memory }
.VAR/DM/ABS=0x3404 scalelist[N]; {initialize with the scale factor for each biquad }
.VAR/DM/ABS=0x3408 X ; {input sample}
.VAR/DM/ABS=0x3409 Y ; {output sample}

.VAR/PM/ABS=0x100 coefflist[N_x_5];  { init with filter coefficients for each biquad }
.INIT		delayline: 0,0,0,0 ; 
.INIT           scalelist: 0x02E900,0x079E00,0x0B0900,0x0CCD00 ;
.INIT           coefflist: 0x200000,0x400000,0x200000,0xF5C800,0x0ED000,
                           0x200000,0x400000,0x200000,0xEBB700,0x227F00,
                           0x200000,0x400000,0x200000,0xE99D00,0x2D3500,
                           0x200000,0x400000,0x200000,0xEC3100,0x304200; 
.INIT		X:0x1 ;
.INIT		Y:0x0 ; 

.ENTRY start ; 

start:  
        I0=^delayline;  M0=1;  L0=0;
        I1=^scalelist;  M1=-3; L1=N;
        I4=^coefflist;  M4=1;  L4=N_x_5;
        I6=^X;          M6=1;  L6=0;
	I7=^Y;          M7=1;  L7=0;
        M2=1;
        M3=-3;
	CNTR=N ;  
 
START_PROFILING:
        SR1=DM(I6,M6) ; 
        DO sections UNTIL CE;
          SE=DM(I1,M2);
          MX0=DM(I0,M0), MY0=PM(I4,M4);                    {get x(n-2), B2}
          MR=MX0*MY0(SS), MX1=DM(I0,M0), MY0=PM(I4,M4);    {get x(n-1), B1}
          MR=MR+MX1*MY0(SS), MY0=PM(I4,M4);                {get         B0}
          MR=MR+SR1*MY0(SS), MX0=DM(I0,M0), MY0=PM(I4,M4); {get y(n-2), A2}
          MR=MR+MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M4); {get y(n-1), A1}
          DM(I0,M0)=MX1, MR=MR+MX0*MY0(RND);   {store x(n-1) as new x(n-2)}
sections: DM(I0,M0)=SR1, SR=ASHIFT MR1 (HI);     {store x(n) as new x(n-1)}
  
        DM(I0,M0)=MX0; {store y(n-1) as new y(n-2)}
        DM(I0,M3)=SR1; {store y(n)   as new y(n-1)}
	DM(I7,M6)=SR1; {store y(n) at output address} 

END_PROFILING:

        SR = ASHIFT SR1 BY 1 (HI);
        
END:	JUMP END ; 


.ENDMOD;

