
;  benchmark program  : biq_N_arc.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description        : benchmarking of an N Sections Cascaded IIR Filter
;
; This routine implements an N cascaded blocks of biquad IIR
; canonic type II filters. Each biquad requires 3 data memory
; locations d(n), d(n-1), d(n-2), and 5 coefficients -a1, -a2,
; b0, b1 and b2.
; 
; For each block :
;	d(n) = x(n) + d(n-1)a1 + d(n-2)a2
; 
;   State variables (low to high data memery):
;	d(n) d(n-1) d(n-2), ..., d(n) d(n-1) d(n-2)
;       Nth biquad          ...  1st biquad
;
;   Coefficient Storage  (low to high program memory):
;	-a2 -a1 b0 b1 b2, ..., -a2 -a1 b0 b1 b2
;       1st biquad        ...   Nth biquad
; 
;   Entry Conditions:
;	AR1 -> d(n-2) of 1st biquad
;	AR2 -> -a2 of 1st biquad
;       AR3 -> input sample (Q15 number)
;       AR4 -> output sample (Q15 number)
;       DP = 0 , PM = 0 , ARP = 3
; 
;  reference          : TMS320C5x User's Guide, pp. 7-42
;
;  func. verification : with sim5x simulator 
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 28-03-94 - assembler reference code creation, profiling

	.file "biq_N_arc.asm"
	.title "N Cascaded Biquad IIR Filters Assembler Reference Code"

	.mmregs

N	.set	4		; number of biquads

	  .sect "states"
STATES	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
     	.word 0
S_END   .word 0

	.sect "coeff"
COEFF	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h
	.word	01000h

	.sect "samples"
X_ADD		.word 	3ff0h
Y_ADD		.word	0

        .sect ".text"

INIT:  				; Setup Variables
	LDP	#0		; DP = 0
	ZPR			; Clear P register
	LAR	AR1,#S_END	; AR1 points to d(n-2) of 1st biquad
	LAR	AR2,#COEFF	; AR2 points to -a2 of 1st biquad
	LAR	AR3,#X_ADD	; AR3 points to input sample
	LAR	AR4,#Y_ADD	; AR4 points to output sample
	MAR	*,AR3		; ARP = AR3
	
START_PROFILING:

	LACC	*,15,AR1	; Get Q15 input
	SPLK	#2,INDX		; Setup Index Register
	SPLK	#N-1,BRCR	; Setup count
				; Begin Computation
	RPTB	ELOOP-1		; repeat for N biquads
LOOP:
	LT	*-,AR2		; T = d(n-2)
	MPYA	*+,AR1		; Acc = x(n), P = -d(n-2)a2
	LTA	*-,AR2		; Acc += -d(n-2)a2, T = d(n-1)
	MPY 	*+		; P = -d(n-1)a1
	LTA	*+,AR1		; Acc += -d(n-1)a1, T = b2
	SACH	*0+,1		; Save d(n)
	MPY	*-		; P = d(n-2)b2
	LACL	#0		; Acc = 0 
	LTD	*-,AR2		; T = d(n-1), d(n-2) = d(n-1)
	MPY	*+,AR1		; Acc += d(n-2)b2 , P = d(n-1)b1
	LTD	*-,AR2		; T = d(n), d(n-1) = d(n)
	MPY	*+,AR1		; Acc += d(n-1)b1, P = d(n)b0
ELOOP:
	LTA	*,AR4		; final accumulation
	SACH	*,1		; Save output in Q15 format

END_PROFILING:

	B END_PROFILING		; wait forever
