{   Coefficient Adaption Subroutine                           LMS_ARC.DSP


   benchmark program  : lms_arc.dsp
 
   benchmark suite    : DSP-kernel
 
   description        : benchmarking of lms adaptive filtering 
                        
   The equations of the filter are:

   e(n)  = d(n) - H(n)  * X(n)         (FIR filter and error)
   H(n+1)= H(n) + delta * X(n) * e(n)  (Coefficient update)
  
 
                x(n-0)  ---- x(n-1)  ----- x(n-i) -----  x(n-N+1)
     x(n) ----->-------| 1/z|-------| 1/z|-.....-| 1/z|----
                   |   -----    |   ------   |   ------   |
                   h0           h1           hi          hN-1
                     \________  |____   _____|  _________/
                              \______(+)_______/
                                      |
                                      |---------> y(n)
                                      v
           d(n) -------------------->(+)--------> e(n)
                                    -
  
  

   reference code     :  Analog Devices' Digital Signal Processing Laboratory 
                         Chapter 5, Digital Filters, p. 5.3.1

 
   func. verification : from separate computation
 
   organization       : Aachen University of Technology - IS2
                      : DSP Tools Group
                      : phone   : +49(241)807887
                      : fax     : +49(241)8888195
                      : e-mail  : zivojnov@ert.rwth-aachen.de
 
   author             : Juan Martinez Velarde
 
   history            : 30-3-1994 creation and profiling

    Calling Parameters
         I0,I2 -> Newest Input Data Value, x(0), in Delay Line
         L0 = Filter Length (N) or Taps
         I4 -> Beginning of Filter Coefficient Table
         L4 = Filter Length (N) or Taps
         M0, M4 = 1
         CNTR = Filter Length - 1 (N - 1)
         I1 -> Desired Output
         I3 -> Delta
         I5 -> Output Address
         I7 -> Input Address

    Return Values
	Coefficients Updated
	I2 -> Oldest Input data value in delay line
	I6 -> Beginning of filter coefficient table

}

.MODULE                      lms ;
.CONST                       N=16 ;          {filter length} ; 
.VAR/PM/RAM/ABS=0x0          coeff[N];
.INIT                        coeff:<coeff.dat> ;
.VAR/DM/RAM/ABS=0x0          states[N];
.VAR/DM/RAM                  desired ; 
.VAR/DM/RAM                  delta  ;
.VAR/DM/RAM                  input;
.VAR/DM/RAM                  output;
.INIT                        states: 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1;
.INIT                        desired: 3 ; 
.INIT                        delta: 1 ; 
.INIT                        input: 2;
.INIT                        output: 0;

.ENTRY lms;

lms_begin:     
         I0=^states;     L0=%coeff; M0=1 ;
         I1=^desired;    L1=0;      M1=1;
         I2=^states+N-1; L2=N;      
         I3=^delta ;     L3=0;      M3=-1;
         I4=^coeff;      L4=%coeff; M4=1;
         I5=^output;     L5=0;      M5=1 ;
         I6=^coeff;      L6=%coeff; M6=2;
         I7=^input;      L7=0;      M7=-1;
         CNTR = N-1 ;

START_PROFILING:

{ first do a fir convolution } 

         MX0=DM(I7,M7) ; {get input sample}
         DM(I0,M0)=MX0 ; {write it as newest state}
         I0=^states+N-1; {I0 points to oldest input data value} 
         MR=0, MX0=DM(I0,M0), MY0=PM(I4,M4); 
         DO conv UNTIL CE;                                 {do fir}
conv:          MX0=DM(I0,M0), MY0=PM(I4,M4), MR=MR+MX0*MY0(SS) ; {convolution}
         MR=MR+MX0*MY0(RND);                               {last step}
         IF MV SAT MR;

         {rounded ans saturated result is now in MR1}
         {calculate error, multiply it with delta}
        
         AF=-MR1,AX0=DM(I1,M1) ;   {negate the result,get desired sample}
         AR=AX0+AF, MY1=DM(I3,M1); {e = d - y, MY1=delta}
         MX0=AR ;                  {MX0=error e}

         {do coefficient update}

         MF=MX0*MY1(RND), MX0=DM(I2,M1) ; {MF=error*delta, get new state}
         MR=MX0*MF(RND),  AY0=PM(I6,M5) ; {MR=error*delta*X, AY0=h0}
         CNTR=N ; DO adapt UNTIL CE ; 
            AR=MR1+AY0,   MX0=DM(I2,M1),  AY0=PM(I6,M7) ; {coeff+=error*delta*X,get new state,new coeff}
adapt:      PM(I6,M6)=AR, MR=MX0*MF(RND) ;                {update coeff, multiply new error*delta*X}
         
         MODIFY(I2,M3);  {point to oldest data}
         MODIFY(I6,M7);  {point to start of table}
         DM(I5,M5)=AR;   {save output value}

END_PROFILING:

end:     JUMP END ; 

.ENDMOD;
