;  benchmark program  : matrix_arc.asm
; 
;  benchmark suite    : DSP-kernel
;
;  description:         [XxY] [YxZ] generic matrix - multiply benchmarking
; 
;          A[X x Y] * B[Y x Z] = C[X x Z]
;                    
;                  |a11     a12     ..      a1y|
;                  |a21     a22     ..      a2y|
; matrix A[X x Y]= |..      ..      ..     ..  |
;                  |a(x-1)1 a(x-1)2 ..  a(x-1)y|
;                  |ax1     ax2     ..      axy|
;
;
;                  |b11     b12     ..     b1z|
;                  |b21     b22     ..     b2z|
; matrix B[Y x Z]= |..      ..      ..     .. |
;                  |b(y-1)1 b(y-1)2 .. b(y-1)z|
;                  |by1     by2     ..     byz|
;
;                  |c11     c12     ..     c1z|
;                  |c21     c22     ..     c2z|
; matrix C[X x Z]= |..      ..      ..     .. |
;                  |c(x-1)1 c(x-1)2 .. c(x-1)z|
;                  |cx1     cx2     ..     cxz|
; 
; matrix elements are stored as
;
; A[X x Y] = { a11, a12, .. , a1y, 
;              a21, a22, .. , a2y, ..., 
;              ax1, ax2, .. , axy}
; 
; B[Y x Z] = { b11, b21, .., b(y-1)1, by1, b12, b22, .. , b(y-1)z, byz} 
; 
; C[X x Z] = { c11, c21, .. , c(x-1)1, cx1, c12, c22, .. ,c(x-1)z, cxz }
; 
; 
;
;  reference :          TI_BBS' c5xug/mtrxmpy.asm (modified)
;                       TMS320C5x User's Guide p.7-11.
;
;  func. verification : from sim5x simulator
;
;  organization       : Aachen University of Technology - IS2
;                     : DSP Tools Group
;                     : phone   : +49(241)807887
;                     : fax     : +49(241)8888195
;                     : e-mail  : zivojnov@ert.rwth-aachen.de
;
;  author             : Juan Martinez Velarde
;
;  history            : 07-04-94 - assembler reference code creation 
;                       07-04-94 - assembler reference code profiling

        .mmregs

        .file  "matrix_arc.asm"
        .title "[XxY][YxZ] matrix multiply assembler reference code"
	.width 120

A_MAT	.set	0fe00h		; memory address of A_MAT in PAGE 0
B_MAT	.set	0300h		; memory address of B_MAT in PAGE 1
C_MAT	.set    0400h		; memory address of C_MAT in PAGE 1

X_DIM	.set 	0ah		; first dimension  of A matrix
Y_DIM	.set	0ah		; second dimension of A matrix, first of B matrix
Z_DIM	.set	0ah		; second dimension of B matrix

; following sections are defined in the cmd file !

; sect matrix_a should be loaded in PAGE 0 

	.sect 	"matrix_a"

	.word 	1,1,1,1,1,1,1,1,1,1
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10

; sect matrix_b and matrix_c should be loaded in PAGE 1 

	.sect	"matrix_b"
	.word 	1,1,1,1,1,1,1,1,1,1
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10
	.word 	1,2,3,4,5,6,7,8,9,10

	.sect	"matrix_c"
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0
	.word	0,0,0,0,0,0,0,0,0,0

        .sect ".text"
INIT:				;program entry point
	LDP	#0
	LAR	AR1,#A_MAT	;POINTS TO A_MAT
	LAR	AR2,#B_MAT	;POINTS TO B_MAT
	LAR	AR3,#C_MAT	;POINTS TO C_MAT
	SPLK	#Y_DIM,INDX	;INDX = Y_DIM 
	MAR	*,AR2		;ARP = AR2 

START_PROFILING:
      	LAR	AR0,#(Z_DIM-1)	;outer loop	
	SAR	AR2,AR4 	;save address of B(0,0) in AR4
LOOP1:	SMMR	AR1,BMAR	;BMAR -> begin of A(i,0)
	SPLK	#(X_DIM-1),BRCR	;RPT BLOCK FOR THE X_DIM ROWS
	SAR	AR4,AR5		;AR5 -> B(0,0)
LOOP2:	RPTB	ELOOP2		;ARP MUST BE AR2 IN THE LOOP
	SAR	AR5,AR2 	;AR2 -> B(0,j)
LOOP3:	RPTZ	#(Y_DIM-1)	;RPT NXT INSTR Y_DIM TIMES PRECEDED 
                                ;BY CLEARING OF ACC AND PREG
ELOOP3:	MADS	*+		;ACC = A(i,k) * B(k,j)
	APAC			;FINAL ACCUMULATION
	MAR	*,AR5		;ARP=AR5
	MAR	*0+,AR3		;UPDATE AR5 BY INDX, ARP=AR3
ELOOP2: SACL	*+,0,AR2	;SAVE Y(J), ARP=AR2
	MAR	*,AR0		;LOOP BACK IF
	BANZD	LOOP1,*-,AR1	;COUNT != #Z_DIM
	ADRK	#Y_DIM		;AR1 -> A(i+1,0)
ELOOP1:	MAR	*,AR2		;ARP=AR2

END_PROFILING:

END:        B END

        .end