;/*
; * benchmark program:   n_complex_updates_arc1.asm
; * 
; * benchmark suite:     DSP-kernel
; * 
; * description:         n_complex_updates - filter benchmarking
; * 
; * This program performs n complex updates of the form 
; *           D(i) = C(i) + A(i)*B(i),
; * where A(i), B(i), C(i) and D(i) are complex numbers,
; * and i = 1,...,N
; *          A(i) = Ar(i) + j Ai(i)
; *          B(i) = Br(i) + j Bi(i)
; *          C(i) = Cr(i) + j Ci(i)
; *          D(i) = C(i) + A(i)*B(i) =   Dr(i)  +  j Di(i)
; *                      =>  Dr(i) = Cr(i) + Ar(i)*Br(i) - Ai(i)*Bi(i)
; *                      =>  Di(i) = Ci(i) + Ar(i)*Bi(i) + Ai(i)*Br(i)
; * 
; * reference code:      target assembly
; * 
; * f. verification:     printf function
; * 
; *  organization:        Aachen University of Technology - IS2 
; *                       DSP Tools Group
; *                       phone:  +49(241)807887 
; *                       fax:    +49(241)8888195
; *                       e-mail: zivojnov@ert.rwth-aachen.de 
; *
; * author:              Juan Martinez Velarde
; * 
; * history:             07-7-94 creation (Martinez Velarde)
; * 
; */
	section n_complex_updates_arc1
	opt	so,nomd,cc


N	equ	16	; number of updates

        org     x:$0
V__a:   
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10
	dc      20
	dc	10

V__b:   
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20
	dc      50	
	dc      20
   	dc      50	
	dc      20
   	dc      50	
	dc      20	
	dc      50	
	dc      20
	dc      50	
	dc      20
	dc      50	
V__c:   
	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40
	dc      30	
	dc      40
   	dc      30	
	dc      40
   	dc      30	
	dc      40	
	dc      30	
	dc      40
	dc      30	
	dc      40

V__d:   
	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0
	dc      0	
	dc      0
   	dc      0	
	dc      0
   	dc      0	
	dc      0	
	dc      0	
	dc      0
	dc      0	
	dc      0

saver2	dc	0

LC0	dc	'dr',$3D,$25,'i',$0A,$0A,$00
LC1	dc	'di',$3D,$25,'i',$0A,$0A,$00

	org	p:
	global	Fmain
Fmain
	move	ssh,x:(r2)+
	move	r2,x:saver2

        move 	#V__a,r0
	move	#V__b,r3
        move 	#V__c,r1
	move	#V__d,r2
	clr	a
	clr	b
	nop

	START_PROFILING:
	
	move	x:(r1)+,a0					;a=cr (first)

	do	#N,END_PROFILING

	asl	a						;shift left
	move	x:(r0)+,y1	x:(r3)+,x1			;y1=ar,x1=br
        mac    	y1,x1,a    	x:(r0)+,y0	x:(r3)+,x0	;accu=cr+ar*br,y0=ai,x0=bi
	mac	-y0,x0,a 	x:(r1)+,b0			;accu-=ai*bi; b=ci
	asr	a						;shift right
	asl	b		a0,x:(r2)+  			;shift left,store dr
	mac	y1,x0,b						;accu=ci+ai*br
	mac	y0,x1,b		x:(r1)+,a0			;accu+=ar*bi
	asr	b						;shift right
	move	b0,x:(r2)+					;store di

	END_PROFILING:

; **************************************************
	

	move	x:saver2,r2

	do	#N,ENDLOOP

	move	#V__d,r0
  	move	#LC0,r1
	clr	a
	move	x:(r0)+,a0
	move	a0,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move	(r2)-
	move	(r2)-

	move	#LC1,r1
	clr	b
	move	x:(r0)+,b0
	move	b0,x:(r2)+
	move	r1,x:(r2)+
	jsr	Fprintf
	move 	(r2)-
	move	(r2)-


ENDLOOP:

; **************************************************
	move	x:-(r2),ssh
	rts

	endsec